; NAME:
;     linemoments
;
; PURPOSE:
;     To compute the moments of an absorption/emission line
;       
; CALLING SEQUENCE:
;     result = linemoments(wave,flux,wmin,wmax,linewave,FERR=ferr,$
;                          MERR=merr,CANCEL=cancel)
;
; INPUTS:
;     wave     - Wavelength array
;     flux     - Flux array
;     wwin     - The lower wavelength limit of the line
;     wmax     - The upper wavelength limit of the line
;     linewave - The rest wavelength of the line
;
; OUTUTS:
;     Returns an array of line moments.
;
;     result[0]  - The zeroth moment of the flux distribution 
;     result[1]  - The first moment of the flux distribution
;     result[2]  - The radial velocity based on result[4]
;     result[3]  - The second moment of the flux distribution
;     result[4]  - The zeroth moment of the net flux (flux-cont)
;     result[5]  - The first moment of the net flux (flux-cont)
;     result[6]  - The radial velocity based on result[8]
;     result[7] - The second moment of the net flux (flux-cont)
;     result[8] - The equivalent width of the line
;     result[9] - The first moment of the continuum flux distribution
;
; KEYWORD PARAMETERS:    
;     FERR   - An error array
;     MERR   - An array of error values for the returned moment array
;     CANCEL - Set on return if there is a problem
;
; PROCEDURE:
;     Based on the IUE program FEATURE.PRO.
;     Note the first moment of the line can be biased if the continuum
;     is sloped.  See Robertson, J.G.  1986, PASP, 98, 122 
;
; MODIFICATION HISTORY:
;     2001 - Written by M. Cushing, Institute for Astronomy, UH
;
function linemoments,wave,flux,wmin,wmax,linewave,FERR=ferr,MERR=merr,$
                     CANCEL=cancel

cancel = 0

if n_params() lt 5 then begin
    
    cancel = 1
    print, 'Syntax - result = linemoments(wave,flux,wmin,wmax,linewave,$'
    print, '                              FERR=ferr,MERR=merr,$'
    print, '                              CONT=cont,CANCEL=cancel'
    return, -1

endif
zparcheck, 'linemoments',wave, 1, [2,3,4,5], 1, 'Wave'
zparcheck, 'linemoments',flux, 2, [2,3,4,5], 1, 'Flux'
zparcheck, 'linemoments',wmin, 3, [2,3,4,5], 0, 'Wmin'
zparcheck, 'linemoments',wmax, 4, [2,3,4,5], 0, 'Wmax'
zparcheck, 'linemoments',linewave, 5, [2,3,4,5], 0, 'Linewave'

ndt = n_elements(wave)

;  Locate the left and right continuum points in channel space.

tabinv,wave,[wmin,wmax],idx
i = round(idx)

;  Determine the continuum level (and its error)

m    = ( flux[i[1]]-flux[i[0]] ) / ( wave[i[1]]-wave[i[0]] )
b    = flux[i[0]] - ( m * wave[i[0]] )
cont = poly(wave,[b,m])

if n_elements(FERR) ne 0 then cerr = replicate((ferr[i[0]]+ferr[i[1]])/2.,ndt)

;  Compute some useful quantities

net   = flux-cont
div   = flux/cont
wflux = flux*wave
wnet  = wave*net

if n_elements(FERR) ne 0 then begin

    enet   = sqrt(ferr^2+cerr^2)
    ediv   = sqrt( (flux/cont)^2*cerr^2 + (1/cont^2)*ferr^2 )
    ewflux = wave*ferr
    ewnet  = wave*enet

endif

;  Determine zeroth moment of the line (Area under the curve).

m0_cont = traper(wave,flux,wmin,wmax,ERROR=cerr,SIG=em0_cont)
m0_flux = traper(wave,flux,wmin,wmax,ERROR=ferr,SIG=em0_flux)
m0_net  = traper(wave,net,wmin,wmax,ERROR=enet,SIG=em0_net)
EW      = traper(wave,(1.-div),wmin,wmax,ERROR=ediv,SIG=eEW)

;  Determine first moment of the line (Position of the line).

m1_flux = traper(wave,wflux,wmin,wmax,ERROR=ewflux,SIG=em1_flux) / m0_flux
rv_flux = (m1_flux-linewave) * 2.997925e5 / linewave
m1_net  = traper(wave,wnet,wmin,wmax,ERROR=ewnet,SIG=em1_net) / m0_net
rv_net  = (m1_net-linewave) * 2.997925e5 / linewave

if n_elements(FERR) ne 0 then begin

    em1_flux = sqrt((m1_flux/m0_flux)^2*em0_flux^2 + (1/m0_flux^2)*em1_flux^2)
    erv_flux = em1_flux * 2.997925e5 / linewave
    em1_net  = sqrt((m1_net/m0_net)^2*em0_net^2 + (1/m0_net^2)*em1_net^2)
    erv_net  = em1_net * 2.997925e5 / linewave

endif

;  Determine second moment of the line (Width of the line).

wwflux = flux*(wave-m1_flux)^2
wwnet  = (flux-cont)*(wave-m1_net)^2

m2_flux = sqrt( abs(traper(wave,wwflux,wmin,wmax)/m0_flux) )
m2_net  = sqrt( abs(traper(wave,wwnet,wmin,wmax)/m0_net) )


;  Return results

if n_elements(FERR) ne 0 then merr = [em0_flux,em1_flux,erv_flux,$
                                      !values.f_nan,em0_net,$
                                      em1_net,erv_net,$
                                      !values.f_nan,eEw,em0_cont]

return,[m0_flux,m1_flux,rv_flux,m2_flux,m0_net,m1_net,rv_net,m2_net,EW,m0_cont]

end
