; NAME:
;     telluric
;
; PURPOSE:
;     To construct the telluric correction spectra for SpeX
;       
; CALLING SEQUENCE:
;     telluric,wave,flux,error,mag,bminv,kernel,scales,wvega,$
;              fcvega,tellcor,tellcor_error,scvega,CANCEL=cancel
;
; INPUTS:
;     std_wave   - The wavelength array of the A0 V standard (in microns)
;     std_flux   - The flux array of the A0 V standard
;     std_error  - The error array of the A0 V standard
;     std_mag    - The magnitude of the A0 V standard
;     std_bminv  - The (B-V) color of the A0 V standard
;     kernel     - The convolution kernel
;     scales     - An array of scale factors for the Vega model at 
;                  the wavelengths "wave"
;     wvega      - The wavelength array of the Vega model shifted to 
;                  radial velocity of the A0 V standard
;     fcvega     - The continuum flux array of the Vega model
;
; OUTUTS:
;     tellcor       - The telluric correction spectrum
;     tellcor_error - The telluric correction error spectrum 
;     scvega        - The convolved and scaled Vega model sampling 
;                     at wave
;
; KEYWORD PARAMETERS:    
;     CANCEL  - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;
; PROCEDURE:
;     Based entirely on W. Vacca's telluric.  
;
; MODIFICATION HISTORY:
;     2002-02-06 - Written by M. Cushing, Institute for Astronomy, UH
;
pro telluric,std_wave,std_flux,std_error,std_mag,std_bminv,kernel,scales,$
             wvega,fvega,cfvega,cf2vega,tellcor,tellcor_error,scvega,$
             CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 11 then begin
    
    print, 'Syntax - telluric,std_wave,std_flux,std_error,std_mag,std_bminv,$'
    print, '                  kernel,scales,wvega,fcvega,fc2vega,tellcor,$'
    print, '                  tellcor_error,scvega,CANCEL=cancel'
    cancel = 1
    return

endif
zparcheck, 'telluric', std_wave, 1, [1,2,3,4,5], 1, 'Wave'
zparcheck, 'telluric', std_flux, 2, [1,2,3,4,5], 1, 'Flux'
zparcheck, 'telluric', std_error, 3, [1,2,3,4,5], 1, 'Error'
zparcheck, 'telluric', std_mag, 4, [1,2,3,4,5], 0, 'Mag'
zparcheck, 'telluric', std_bminv, 5, [1,2,3,4,5], 0, 'Bminv'
zparcheck, 'telluric', kernel, 6, [1,2,3,4,5], 1, 'Kernel'
zparcheck, 'telluric', scales, 7, [1,2,3,4,5], 1, 'Scales'
zparcheck, 'telluric', wvega, 8, [1,2,3,4,5], 1, 'Wvega'
zparcheck, 'telluric', cfvega, 9, [1,2,3,4,5], 1, 'Cfvega'
zparcheck, 'telluric', cf2vega, 10, [1,2,3,4,5], 1, 'Cf2vega'

;  Make sure kernel is normalized.

kernel = kernel/total(kernel)

;  Determine the range over which to convolve the Vega model

wmin = min(std_wave,/NAN,max=wmax)
zv   = where(wvega gt wmin and wvega lt wmax,count)

nkern = n_elements(kernel)
nadd  = round(nkern/2.)

idx = findgen(count+2*nadd)+(zv[0]-nadd)

;  Interpolate the scale array onto the Vega sampling 

linterp,std_wave,scales,wvega[idx],rscales

;  Now do the convolution

nfvconv = convol( (fvega[idx]/cfvega[idx]-1.0) * rscales, kernel ) + 1.0
cfvconv = convol( (cfvega[idx]/cf2vega[idx]-1.0) * rscales, kernel ) + 1.0
cfvconv = temporary(cfvconv)*cf2vega[idx]
fvconv  = nfvconv*cfvconv

;  Now interpolate wvconv and fvconv onto the sampling of the A0 V

linterp,wvega[idx],fvconv,std_wave,rfvconv

;  Determine scale factors for flux calibration and reddening

vegamag  = 0.03
vegabmv  = 0.00
magscale = 10.0^(-0.4*(std_mag-vegamag))
ebmv     = (std_bminv - vegabmv) > 0.0 ; to prevent reddening the spectrum
avred    = 3.10*ebmv

;  Redden the convolved Vega model

ccm_unred, std_wave*10000.0, rfvconv, -1.0*ebmv, fvred, R_V=rvred

;  Scale to the observed V mag of the A0V star

fvred = temporary(fvred)*( magscale*(10.0^(0.4*avred)) )

scvega = fvred

;  Calculate the ratio: model/observations


z  = where(finite(std_flux) eq 1,count)
wa = std_wave[z]
fa = std_flux[z]
ea = std_error[z]
fv = fvred[z]

z  = where(fa gt 0.0D)
wa = wa[z]
fa = fa[z]
ea = ea[z]
fv = fv[z]



t  = fv/fa
te = sqrt( (fv/fa^2)^2 * ea^2 )


interpspec,wa,t,std_wave,tellcor,tellcor_error,YAERROR=te

end


