; NAME:
;     traper
;
; PURPOSE:
;     To integrate a sampled function with errors.
;       
; CALLING SEQUENCE:
;
;
; INPUTS:
;     wave     - Wavelength array
;     flux     - Flux array
;     wwin     - The lower wavelength limit of the line
;     wmax     - The upper wavelength limit of the line
;
; OUTUTS:
;     Returns the integrated value
;
; KEYWORD PARAMETERS:    
;     ERROR  - An error array.  If given, the error on the 
;              integration is given in SIG.
;     SIG    - If ERRROR is given, this is the propagated error 
;              on the integration.
;     CANCEL - Set on return if there is a problem
;
; PROCEDURE:
;     Theory is based on the program TRAPER in the Mathlib 
;     FORTRAN package used at CERN.
;      
; MODIFICATION HISTORY:
;     2002-04-10 - Written by M. Cushing, Institute for Astronomy, UH
;
function traper,wave,flux,wmin,wmax,ERROR=error,SIG=sig,CANCEL=cancel

cancel = 0

if n_params() lt 4 then begin
    
    cancel = 1
    print, 'Syntax - result = traper(wave,flux,wmin,wmax,ERROR=error,$'
    print, '                         SIG=sig,CANCEL=cancel)'
    return, -1

endif
zparcheck, 'traper',wave, 1, [2,3,4,5], 1, 'Wave'
zparcheck, 'traper',flux, 2, [2,3,4,5], 1, 'Flux'
zparcheck, 'traper',wmin, 3, [2,3,4,5], 0, 'Wmin'
zparcheck, 'traper',wmax, 4, [2,3,4,5], 0, 'Wmax'

;  Get values of the function at the required points.

z     = where(wave gt wmin and wave lt wmax,count)
ww    = [wmin,wave[z],wmax]
nwave = ww[uniq(ww)]
ndat  = count+2

interpspec,wave,flux,ww,nflux,nerror,YAERROR=error

;  Determine the delta lambda between each point and then the weights

dwave = (shift(nwave,-1)-nwave)[0:ndat-2]
weights = [dwave[0],(shift(dwave,-1)+dwave)[0:ndat-3],dwave[ndat-2]]/2.

;  Perform the integration

sum = total(weights*nflux)

if n_elements(ERROR) ne 0 then SIG=sqrt(total(weights^2 * nerror^2))

return, sum

end




