;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NAME:
;     VEGACORR.PRO
;
; PURPOSE:
;     To cross correlate a spectrum against the Vega model 
;
; MODIFICATION HISTORY:
;     Written 04 Dec. 2000 by WDV
;
; CALLING SEQUENCE:
;     vegacorr,wave,flux,wvin,fvin,fcvin,vshift,z,wvega,fvega,fcvega,fnvega,$
;               WMIN=wmin,WMAX=wmax,PLOTCORR=plotcorr
;
; INPUTS:
;     wave      - An array of wavelength values (in microns)
;     flux      - An array of normalized flux values
;     wvin      - The wavelength array for the Vega model
;     fvin      - The flux array for the Vega model
;     cfvin     - The continuum fluxes for the Vega model
; 
; OUTPUTS:
;     vshift    - Velocity shift in km/s for which the correlation 
;                 is a maximum
;     z         - The redshift for the vshift
;     wvega     - Array of shifted wavelengths for the Vega model (in microns)
;     fvega     - Array of Vega model fluxes
;     fcvega    - Array of continuum fluxes for Vega model
;     fnvega    - Array of normalized fluxes for Vega model
;
; KEYWORD PARAMETERS:    
;     WMIN      - Wavelength minimum for the correlation
;     WMAX      - Wavelength maximum for the correlation
;     PLOTCORR  - Set if the cross-correlation function is to be
;                 plotted
;
; PROCEDURES CALLED:
;     c_correlate 
;     interpol
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro vegacorr, wave,flux,wvin,fvin,fcvin,vshift,z,$
               WMIN=wmin,WMAX=wmax,PLOTCORR=plotcorr


dum      = ' '
cspeed   = 2.99792458E5 		    ; km/s

wvega  = wvin
fvega  = fvin                     ; original A0V model fluxes 
fnvega = (fvin/fcvin) - 1.0    ; normalize and subtract normalization
fcvega = fcvin
fdata  = flux - 1.0                  ; subtract normalization

; --- Find min and max wavelengths

if keyword_set(WMIN) then begin
      wmin = max([wmin,min(wvega),min(wave)])
endif else begin
    wmin = max([min(wvega),min(wave)])
endelse

if keyword_set(WMAX) then begin
      wmax = min([wmax,max(wvega),max(wave)])
endif else begin
    wmax = min([max(wvega),max(wave)])
endelse

print, 'Cross correlating between: ', wmin, wmax

; --- Check number of points

ndata = n_elements(where((wave  ge wmin) and (wave  le wmax)))
nvega = n_elements(where((wvega ge wmin) and (wvega le wmax)))
if (nvega lt ndata) then print, 'Warning: Input data has higher resolution than Vega model!'
num   = max([ndata,nvega])

; --- Resample to constant spacing in log lambda space: v/c = d(ln lambda)

acoef  = float(num - 1)/(alog(wmax) - alog(wmin))
bcoef  = float(num) - (acoef * (alog(wmax)))

xpon   = findgen(num)+1.0
wx     = exp((xpon-bcoef)/acoef)

fxdata = interpol(fdata,wave,wx)
fxvega = interpol(fnvega,wvega,wx)

; --- Perform cross correlation

lag    = indgen(num)-(num/2)
result = c_correlate(fxvega,fxdata,lag)
pfit   = mpfitpeak(lag,result,pcoefs,/GAUSSIAN,NTERMS=3)
lshift = pcoefs(1)

;maxcor = max(result)
;lshift = lag(where(result eq maxcor))
vshift = (cspeed * lshift)/acoef
z      = (vshift/cspeed)
rshift = 1.0 + z
print, 'Velocity shift (km/s) = ',vshift
print, 'Redshift term         = ',rshift

; --- Plot cross correlation function versus lag

if keyword_set(PLOTCORR) then begin
      window,/free
      plot, lag, result, xtitle='Lag', ytitle='Cross Correlation'
endif

; --- Shift Vega model

rvec   = replicate(rshift,n_elements(wvega))
wvega  = wvega * rvec                   ; shift wavelengths
fnvega = fnvega + 1.0                   ; restore normalization

end









