; NAME:
;     xfindshift
;
; PURPOSE:
;     To determine the shift between and object and telluric spectrum.
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     xfindshift,objwave,objflux,stdflux,shift,OBJERROR=objerror,$
;               STDERROR=stderror,YTITLE=ytitle,XTITLE=xtitle,$
;               INITSHIFT=initshift
;
; INPUTS:
;     objwave - The object wavelength array.
;     objflux - The object flux array.
;     stdflux - The standard flux array (same wavelength sampling as obj).
;
; OUTUTS:
;     shift - The shift in pixels between the spectra.
;
; KEYWORD PARAMETERS:    
;     OBJERROR  - The object error array.
;     STDERROR  - The standard error array.
;     YTITLE    - The ytitle of the plot.
;     XTITLE    - The xtitle of the plot.
;     INITSHIFT - An initial pixel shift.
;
; PROCEDURE:
;     Automatically or manually determines the shift using a chi^2 m
;     minimization technique.
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xfindshift_initcommon

common xfindshift_state, state

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.

w = {keyboard:0L,$
     message:0L,$
     plotwin1:0,$
     plotwin2:0,$
     scale_fld:[0L,0L],$
     shift_fld:[0L,0L],$
     xfindshift_base:0L,$
     xmin_fld:[0L,0L],$
     xmax_fld:[0L,0L],$
     ymin1_fld:[0L,0L],$
     ymax1_fld:[0L,0L],$
     ymin2_fld:[0L,0L],$
     ymax2_fld:[0L,0L]}

r = {cancel:0,$
     cursormode:'None',$
     reg:[!values.f_nan,!values.f_nan],$
     noerror:0L,$
     fix:0,$
     scale:1.,$
     shift:0.}

d = {residspec:ptr_new(fltarr(2)),$
     objerror:ptr_new(fltarr(2)),$
     objflux:ptr_new(fltarr(2)),$
     objwave:ptr_new(fltarr(2)),$
     scstderror:ptr_new(fltarr(2)),$
     scstdflux:ptr_new(fltarr(2)),$
     shstderror:ptr_new(fltarr(2)),$
     shstdflux:ptr_new(fltarr(2)),$
     stderror:ptr_new(fltarr(2)),$
     stdflux:ptr_new(fltarr(2))}


p = {activespec:1,$
     buffer:[0.,0.],$
     cursor:0,$
     pixmap1_wid:0L,$
     pixmap2_wid:0L,$
     plotwin1_wid:0L,$
     plotwin2_wid:0L,$
     plot1xrange:[0.,0.],$
     plot1yrange:[0.,0.],$
     plot1size:[700,300],$
     plot2yrange:[0.,0.],$
     plot2size:[700,214],$
     pscale1:!p,$
     xscale1:!x,$
     pscale2:!p,$
     xscale2:!x,$
     xtitle:'',$
     yscale1:!y,$
     yscale2:!y,$
     ytitle:'',$
     reg:[[!values.f_nan,!values.f_nan],$
          [!values.f_nan,!values.f_nan]]}


;  Load the three structures in the state structure.

state = {w:w,r:r,d:d,p:p}

end
;
;******************************************************************************
;
pro xfindshift_autofind

common xfindshift_state

z = where(*state.d.objwave gt state.r.reg[0] and *state.d.objwave lt $
          state.r.reg[1])

state.r.scale = median((*state.d.objflux)[z]/(*state.d.stdflux)[z],/EVEN)

widget_control, state.w.scale_fld[1],$
  SET_VALUE=string(state.r.scale,format='(f6.3)')
xfindshift_scalespec
xfindshift_shiftspec

shifts = findgen(81)/20.+(state.r.shift-2.)
chisq  = fltarr(81)

for i = 0, 80 do begin

    state.r.shift = shifts[i]
    widget_control, state.w.shift_fld[1],$
      SET_VALUE=string(state.r.shift,format='(f6.3)')
    xfindshift_shiftspec
    xfindshift_plotupdate

    if state.r.noerror then begin
        
        chi2 = total((*state.d.residspec)[z]^2,/NAN) 
        
    endif else begin
       
        chi2 = (*state.d.residspec)[z]^2/$
          ((*state.d.shstderror)[z]^2 + (*state.d.objerror)[z]^2)
        chi2 = total(chi2,/NAN)
        
    endelse
    chisq[i] = chi2 

endfor

min = min(chisq,minidx)

state.r.shift = shifts[minidx]
widget_control, state.w.shift_fld[1],$
  SET_VALUE=string(state.r.shift,format='(f6.3)')

xfindshift_shiftspec
xfindshift_plotupdate
xfindshift_getstat
xfindshift_setminmax

end
;
;******************************************************************************
;
pro xfindshift_getstat

common xfindshift_state

z = where(finite(state.r.reg) eq 1,count)
if count eq 0 then goto, cont

;  Get Chi^2

z = where(*state.d.objwave gt state.r.reg[0] and *state.d.objwave lt $
          state.r.reg[1])

if state.r.noerror then begin

    chi2 = total((*state.d.residspec)[z]^2,/NAN) 

endif else begin

    chi2 = (*state.d.residspec)[z]^2/$
      ((*state.d.shstderror)[z]^2 + (*state.d.objerror)[z]^2)
    chi2 = total(chi2,/NAN)

endelse
widget_control, state.w.message, SET_VALUE='Chi^2 = '+strtrim(chi2,2)

cont:

end
;
;******************************************************************************
;
pro xfindshift_plotresid

common xfindshift_state

plot,*state.d.objwave,*state.d.residspec,/xsty,/ysty,$
  xrange=state.p.plot1xrange,yrange=state.p.plot2yrange,psym=10,$
  xtitle=state.p.xtitle,ytitle='!5Residuals'


z = where(finite(state.r.reg) eq 1,count)
if count eq 0 then goto, cont

plots, !x.crange,[0,0],linestyle=1,color=7

for i = 0, count-1 do plots,[state.r.reg[i],state.r.reg[i]],!y.crange,$
  color=2,linestyle=1,thick=2

state.p.xscale2 = !x
state.p.yscale2 = !y
state.p.pscale2 = !p

cont:

end
;
;******************************************************************************
;
pro xfindshift_plotsetup,objwave,objflux,objerror,stdflux,stderror

common xfindshift_state

*state.d.objwave = objwave

;  Normalize the object spectrum.

moments,objflux,mean
*state.d.objflux = objflux/mean

;  Scale the std to the new obj level.

;div = *state.d.objflux/stdflux
;scale = median(div,/EVEN)

scale = robustsfactor(*state.d.objflux,stdflux,3,0.1,CANCEL=cancel)
if cancel then return

*state.d.stdflux  = stdflux*scale
*state.d.stderror = stderror*sqrt(scale)
*state.d.scstdflux  = stdflux*scale
*state.d.scstderror = stderror*sqrt(scale)

;  Get ranges.

state.p.plot1xrange = [min(*state.d.objwave,/NAN,max=max),max]
state.r.reg         = state.p.plot1xrange

ymin = 0
ymax = max([max(*state.d.objflux,/NAN),max(*state.d.stdflux,/NAN)])
state.p.plot1yrange = [ymin,ymax]

xfindshift_shiftspec
xfindshift_getstat
xfindshift_plotupdate
xfindshift_setminmax

end
;
;******************************************************************************
;
pro xfindshift_plotspec

common xfindshift_state

plot,*state.d.objwave,*state.d.objflux,/xsty,/ysty,$
  yrange=state.p.plot1yrange,xrange=state.p.plot1xrange,psym=10,$
  xtitle=state.p.xtitle,ytitle=state.p.ytitle,title=''
oplot,*state.d.objwave,*state.d.shstdflux,color=3,psym=10

legend,['!5Object Spectrum','!5Telluric Spectrum'],textcolor=[1,3],$
  /RIGHT,BOX=0,charsize=str_size('!5TElluric Spectrum',0.3)

z = where(finite(state.r.reg) eq 1,count)
if count eq 0 then goto, cont

for i = 0, count-1 do plots,[state.r.reg[i],state.r.reg[i]],!y.crange,$
  color=2,linestyle=1,thick=2

state.p.xscale1 = !x
state.p.yscale1 = !y
state.p.pscale1 = !p
state.p.cursor = 1

cont:

end
;
;******************************************************************************
;
pro xfindshift_plotupdate

common xfindshift_state

if not state.r.fix then begin

    z = where(*state.d.objwave lt state.p.plot1xrange[1] and $
              *state.d.objwave gt state.p.plot1xrange[0])
  state.p.plot2yrange = [min((*state.d.residspec)[z],/NAN,max=max),max]

endif 

wset, state.p.plotwin1_wid
erase
xfindshift_plotspec 

wset, state.p.pixmap1_wid
erase
xfindshift_plotspec 

wset, state.p.plotwin2_wid
erase
xfindshift_plotresid

wset, state.p.pixmap2_wid
erase 
xfindshift_plotresid

end
;
;******************************************************************************
;
pro xfindshift_scalespec

common xfindshift_state

*state.d.scstdflux  = (*state.d.stdflux)*state.r.scale
*state.d.scstderror = (*state.d.stderror)*sqrt(abs(state.r.scale))

end
;
;******************************************************************************
;
pro xfindshift_setminmax

common xfindshift_state

widget_control, state.w.xmin_fld[1],set_value=strtrim(state.p.plot1xrange[0],2)
widget_control, state.w.xmax_fld[1],set_value=strtrim(state.p.plot1xrange[1],2)
widget_control, state.w.ymin1_fld[1],$
  set_value=strtrim(state.p.plot1yrange[0],2)
widget_control, state.w.ymax1_fld[1],$
  set_value=strtrim(state.p.plot1yrange[1],2)

widget_control, state.w.ymin2_fld[1],$
  set_value=strtrim(state.p.plot2yrange[0],2)
widget_control, state.w.ymax2_fld[1],$
  set_value=strtrim(state.p.plot2yrange[1],2)

end
;
;******************************************************************************
;
pro xfindshift_shiftspec

common xfindshift_state

;  Shift std spectrum.

x1 = findgen(n_elements(*state.d.scstdflux))+state.r.shift
x2 = findgen(n_elements(*state.d.scstdflux))

interpspec,x1,*state.d.scstdflux,x2,y,error,YAERROR=*state.d.scstderror

*state.d.shstdflux  = y
*state.d.shstderror = error

;  Divide spectra.

*state.d.residspec = *state.d.shstdflux-*state.d.objflux

end
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xfindshift_event, event

common xfindshift_state

widget_control, event.id,  GET_UVALUE = uvalue

case uvalue of

    'Accept': widget_control, event.top, /DESTROY

    'Auto Find': xfindshift_autofind

    'Fix': state.r.fix = event.select

    'Keyboard': begin

        case strtrim(event.ch,2) of 
            
            'c': begin
                
                state.r.cursormode = 'None'
                state.p.reg      = [[!values.f_nan,!values.f_nan],$
                                    [!values.f_nan,!values.f_nan]]
                xfindshift_plotupdate
            
            end

            'w': begin

                state.p.plot1xrange = [min(*state.d.objwave,/NAN,max=max),max]
                
                ymin = 0
                ymax = max([max(*state.d.objflux,/NAN),$
                            max(*state.d.stdflux,/NAN)])
                state.p.plot1yrange = [ymin,ymax]
                xfindshift_plotupdate
                xfindshift_setminmax
                
            end

            's': begin

                state.r.cursormode = 'Select'
                state.p.reg = [[!values.f_nan,!values.f_nan],$
                               [!values.f_nan,!values.f_nan]]
                state.r.reg = [!values.f_nan,!values.f_nan]
                xfindshift_plotupdate

            end

            'z': begin

                state.r.cursormode = 'Zoom'
                state.p.reg = [[!values.f_nan,!values.f_nan],$
                               [!values.f_nan,!values.f_nan]]
                xfindshift_plotupdate
                
            end
        
            else:

        endcase

    end

    'Cancel': begin

        state.r.cancel = 1
        widget_control, event.top, /DESTROY

    end
    'Shift': begin

        val = cfld(state.w.shift_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        state.r.shift = val
        xfindshift_shiftspec
        xfindshift_plotupdate
        xfindshift_getstat
        xfindshift_setminmax

    end

    'Scale': begin
        
        val = cfld(state.w.scale_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then goto, cont
        state.r.scale = val
        xfindshift_scalespec
        xfindshift_shiftspec
        xfindshift_plotupdate
        xfindshift_getstat
        xfindshift_setminmax

    end

    'Scale +': begin

        state.r.scale = state.r.scale+0.005
        widget_control, state.w.scale_fld[1],$
          SET_VALUE=string(state.r.scale,format='(f6.3)')
        xfindshift_scalespec
        xfindshift_shiftspec
        xfindshift_plotupdate
        xfindshift_getstat
        xfindshift_setminmax

    end

    'Scale -': begin

        state.r.scale = state.r.scale-0.005
        widget_control, state.w.scale_fld[1],$
          SET_VALUE=string(state.r.scale,format='(f6.3)')
        xfindshift_scalespec
        xfindshift_shiftspec
        xfindshift_plotupdate
        xfindshift_getstat
        xfindshift_setminmax

    end

    'Shift +': begin
        
        state.r.shift = state.r.shift+0.1
        widget_control, state.w.shift_fld[1],$
          SET_VALUE=string(state.r.shift,format='(f7.3)')
        xfindshift_shiftspec
        xfindshift_plotupdate       
        xfindshift_setminmax
        xfindshift_getstat

    end

    'Shift -': begin

        state.r.shift = state.r.shift-0.1
        widget_control, state.w.shift_fld[1],$
          SET_VALUE=string(state.r.shift,format='(f7.3)')
        xfindshift_shiftspec
        xfindshift_plotupdate      
        xfindshift_setminmax
        xfindshift_getstat

    end
    
    else:

endcase

cont: 

end
;
;******************************************************************************
;
pro xfindshift_plotwinevent1, event

common xfindshift_state

widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0

    wset, state.p.plotwin1_wid
    device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
                  state.p.pixmap1_wid]

    wset, state.p.plotwin2_wid
    device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
                  state.p.pixmap2_wid]
    
    goto, cont
    
endif



;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale1
!x = state.p.xscale1
!y = state.p.yscale1
x  = event.x/float(state.p.plot1size[0])
y  = event.y/float(state.p.plot1size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

if event.type eq 1 then begin
            
    if state.r.cursormode eq 'Zoom' then begin

        z = where(finite(state.p.reg) eq 1,count)
        if count eq 0 then state.p.reg[*,0] = xy[0:1] else begin 
            
            state.p.reg[*,1] = xy[0:1]
            state.p.plot1xrange   = [min(state.p.reg[0,*],max=max),max]
            state.p.plot1yrange   = [min(state.p.reg[1,*],max=max),max]
            xfindshift_shiftspec
            xfindshift_plotupdate
            state.r.cursormode   = 'None'
            state.p.reg      = [[!values.f_nan,!values.f_nan],$
                                [!values.f_nan,!values.f_nan]]
            xfindshift_setminmax
            
        endelse
    
    endif 
    if state.r.cursormode eq 'Select' then begin

        z = where(finite(state.r.reg) eq 1,count)
        if count eq 0 then begin

            state.r.reg[0] = xy[0]
            xfindshift_shiftspec
            xfindshift_plotupdate

        endif else begin 
            
            state.r.reg[1] = xy[0]
            xfindshift_shiftspec
            xfindshift_plotupdate
            xfindshift_getstat
            state.r.cursormode   = 'None'
            state.p.reg      = [[!values.f_nan,!values.f_nan],$
                                [!values.f_nan,!values.f_nan]]

        endelse
    
    endif

endif

;  Copy the pixmaps and draw the lines.

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]
plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE


wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]
plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE

if state.r.cursormode eq 'Zoom' then begin

    wset, state.p.plotwin1_wid
    xy = convert_coord(event.x,event.y,/DEVICE,/TO_DATA)
    plots,[state.p.reg[0,0],state.p.reg[0,0]],[state.p.reg[1,0],xy[1]],$
      linestyle=2,color=2
    plots, [state.p.reg[0,0],xy[0]],[state.p.reg[1,0],state.p.reg[1,0]],$
      linestyle=2,color=2

endif

;  Update cursor.

;if state.p.cursor then begin
    
;    tabinv, *state.d.objwave,xy[0],idx
;    idx = round(idx)
;    label = 'Spectrum X: '+strtrim(xy[0],2)+'
;    widget_control,state.w.message,set_value=label
    
;endif

cont:
    
end
;
;******************************************************************************
;
pro xfindshift_plotwinevent2, event

common xfindshift_state

widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0

    wset, state.p.plotwin1_wid
    device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
                  state.p.pixmap1_wid]

    wset, state.p.plotwin2_wid
    device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
                  state.p.pixmap2_wid]
    goto, cont
    
endif

;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale2
!x = state.p.xscale2
!y = state.p.yscale2
x  = event.x/float(state.p.plot2size[0])
y  = event.y/float(state.p.plot2size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

if event.type eq 1 and state.r.cursormode eq 'Zoom' then begin
            
    z = where(finite(state.p.reg) eq 1,count)
    if count eq 0 then state.p.reg[*,0] = xy[0:1] else begin 
        
        state.p.reg[*,1] = xy[0:1]
        state.p.plot2yrange = [min(state.p.reg[1,*],max=max),max]
        xfindshift_shiftspec
        xfindshift_plotupdate
        state.r.cursormode   = 'None'
        state.p.reg      = [[!values.f_nan,!values.f_nan],$
                            [!values.f_nan,!values.f_nan]]
        xfindshift_setminmax
        
    endelse 

endif

;  Copy the pixmaps and draw the lines.

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]
plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE

wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]
plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE
plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,/DEVICE

;  Plot zoom line

if state.r.cursormode eq 'Zoom' then begin

    wset, state.p.plotwin2_wid
    plots,!x.crange,[state.p.reg[1,0],state.p.reg[1,0]],linestyle=2,color=2

endif

;  Update cursor.

;if state.p.cursor then begin
    
;    tabinv, *state.d.objwave,xy[0],idx
;    idx = round(idx)
;    label = 'Cursor X: '+strtrim(xy[0],2)+'
;    widget_control,state.w.message,set_value=label
    
;endif

cont:

end
;
;******************************************************************************
;
pro xfindshift_minmax,event

common xfindshift_state
widget_control, event.id,  GET_UVALUE = uvalue


case uvalue of 

    'X Min': begin

        xmin = cfld(state.w.xmin_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        xmin2 = crange(xmin,state.p.plot1xrange[1],'X Min',/KLT,$
                       WIDGET_ID=state.w.xfindshift_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control, state.w.xmin_fld[0],set_value=state.p.plotxrange[0]
            return
            
        endif else state.p.plot1xrange[0] = xmin2

    end
    'X Max': begin

        xmax = cfld(state.w.xmax_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        xmax2 = crange(xmax,state.p.plot1xrange[0],'X Max',/KGT,$
                       WIDGET_ID=state.w.xfindshift_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control, state.w.xmax_fld[0],set_value=state.p.plotxrange[1]
            return
            
        endif else state.p.plot1xrange[1] = xmax2

    end
    'Y1 Min': begin

        ymin = cfld(state.w.ymin1_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymin2 = crange(ymin,state.p.plot1yrange[1],'Y Min',/KLT,$
                       WIDGET_ID=state.w.xfindshift_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymin1_fld[0],set_val=state.p.plot1yrange[0]
            return
            
        endif else state.p.plot1yrange[0] = ymin2
        
    end
    'Y1 Max': begin

        ymax = cfld(state.w.ymax1_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymax2 = crange(ymax,state.p.plot1yrange[0],'Y Max',/KGT,$
                       WIDGET_ID=state.w.xfindshift_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymax1_fld[0],set_val=state.p.plot1yrange[1]
            return
            
        endif else state.p.plot1yrange[1] = ymax2
        
    end
    'Y2 Min': begin

        ymin = cfld(state.w.ymin2_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymin2 = crange(ymin,state.p.plot2yrange[1],'Y Min',/KLT,$
                       WIDGET_ID=state.w.xfindshift_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymin2_fld[0],set_val=state.p.plot2yrange[0]
            return
            
        endif else state.p.plot2yrange[0] = ymin2
        
    end
    'Y2 Max': begin

        ymax = cfld(state.w.ymax2_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymax2 = crange(ymax,state.p.plot2yrange[0],'Y Max',/KGT,$
                       WIDGET_ID=state.w.xfindshift_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymax2_fld[0],set_val=state.p.plot2yrange[1]
            return
            
        endif else state.p.plot2yrange[1] = ymax2
        
    end
    
endcase

if uvalue eq 'X Min' or uvalue eq 'X Max' then xfindshift_shiftspec
xfindshift_plotupdate
xfindshift_setminmax

end
;
;******************************************************************************
;
pro xfindshift_resize, event

common xfindshift_state

widget_control, state.w.xfindshift_base, tlb_get_size = size

;  Window 1

state.p.plot1size[0]=size[0]-state.p.buffer[0]
state.p.plot1size[1]=(size[1]-state.p.buffer[1])*1.4/2.4

widget_control, state.w.plotwin1, draw_xsize=state.p.plot1size[0]
widget_control, state.w.plotwin1, draw_ysize=state.p.plot1size[1]

wdelete,state.p.pixmap1_wid
window, /free, /pixmap,xsize=state.p.plot1size[0],ysize=state.p.plot1size[1]
state.p.pixmap1_wid = !d.window

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

;  Window 2

state.p.plot2size[0]=size[0]-state.p.buffer[0]
state.p.plot2size[1]=(size[1]-state.p.buffer[1])/2.4

widget_control, state.w.plotwin2, draw_xsize=state.p.plot2size[0]
widget_control, state.w.plotwin2, draw_ysize=state.p.plot2size[1]

wdelete,state.p.pixmap2_wid
window, /free, /pixmap,xsize=state.p.plot2size[0],ysize=state.p.plot2size[1]
state.p.pixmap2_wid = !d.window

xfindshift_shiftspec
xfindshift_plotupdate

end
;
;******************************************************************************
;
; ------------------------------Main Program-------------------------------- 
;
;******************************************************************************
;

;
;******************************************************************************
;
pro xfindshift,objwave,objflux,stdflux,shift,OBJERROR=objerror,$
               STDERROR=stderror,YTITLE=ytitle,XTITLE=xtitle,$
               INITSHIFT=initshift,CANCEL=cancel

common xfindshift_state

zparcheck, 'xfindshift', objwave, 1, [2,3,4,5], 1, 'Std Wave' 
zparcheck, 'xfindshift', objflux, 2, [2,3,4,5], 1, 'Std Flux' 
zparcheck, 'xfindshift', stdflux, 3, [2,3,4,5], 1, 'Obj Flux' 


if n_params() lt 4 then begin

    print, 'Syntax - xfindshift,objwave,objflux,stdflux,shift,$'
    print, '                    OBJERROR=objerror,STDERROR=stderror,$'
    print, '                    YTITLE=ytitle,XTITLE=xtitle,$'
    print, '                    INITSHIFT=initshift,CANCEL=cancel'
    return

endif

if not xregistered('xfindshift') then begin
        
    xfindshift_initcommon
    if n_elements(INITSHIFT) ne 0 then state.r.shift = initshift

;  Build the widget.

    buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'
    textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
    
    state.w.xfindshift_base = widget_base(title='Xfindshift', $
                                         /COLUMN,$
                                         /TLB_SIZE_EVENTS)
    
       quit_button = widget_button(state.w.xfindshift_base,$
                                   FONT=buttonfont,$
                                   EVENT_PRO='xfindshift_event',$
                                   VALUE='Cancel',$
                                   UVALUE='Cancel')
       
       state.w.keyboard = widget_text(state.w.xfindshift_base, $
                                      /ALL_EVENTS, $
                                      SCR_XSIZE=1, $
                                      SCR_YSIZE=1, $
                                      UVALUE='Keyboard', $
                                      EVENT_PRO='xfindshift_event',$
                                      VALUE= '')
       
       row_base = widget_base(state.w.xfindshift_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)

          auto = widget_button(row_base,$
                               FONT=buttonfont,$
                               EVENT_PRO='xfindshift_event',$
                               VALUE='Auto Find',$
                               UVALUE='Auto Find')
             

          shift = coyote_field2(row_base,$
                                LABELFONT=buttonfont,$
                                FIELDFONT=textfont,$
                                TITLE='Shift:',$
                                UVALUE='Shift',$
                                XSIZE=12,$
                                VALUE=state.r.shift,$
                                EVENT_PRO='xfindshift_event',$
                                /CR_ONLY,$
                                TEXTID=textid)
          state.w.shift_fld = [shift,textid]
       
          minus = widget_button(row_base,$
                                FONT=buttonfont,$
                                EVENT_PRO='xfindshift_event',$
                                VALUE=' - ',$
                                UVALUE='Shift -')

          plus = widget_button(row_base,$
                               FONT=buttonfont,$
                               EVENT_PRO='xfindshift_event',$
                               VALUE=' + ',$
                               UVALUE='Shift +')
          
          scale = coyote_field2(row_base,$
                                LABELFONT=buttonfont,$
                                FIELDFONT=textfont,$
                                TITLE='Scale:',$
                                UVALUE='Scale',$
                                XSIZE=12,$
                                VALUE=state.r.scale,$
                                EVENT_PRO='xfindshift_event',$
                                /CR_ONLY,$
                                TEXTID=textid)
          state.w.scale_fld = [scale,textid]
          
          minus = widget_button(row_base,$
                                FONT=buttonfont,$
                                EVENT_PRO='xfindshift_event',$
                                VALUE=' - ',$
                                UVALUE='Scale -')
          
          plus = widget_button(row_base,$
                               FONT=buttonfont,$
                               EVENT_PRO='xfindshift_event',$
                               VALUE=' + ',$
                               UVALUE='Scale +')
          
          state.w.message = widget_label(row_base,$
                                         VALUE='Chi^2 = ',$
                                         FONT=buttonfont,$
                                         /DYNAMIC_RESIZE)

      
       state.w.plotwin1 = widget_draw(state.w.xfindshift_base,$
                                      XSIZE=state.p.plot1size[0],$
                                      YSIZE=state.p.plot1size[1],$
                                      /TRACKING_EVENTS,$
                                      /BUTTON_EVENTS,$
                                      /MOTION_EVENTS,$
                                      EVENT_PRO='xfindshift_plotwinevent1',$
                                      UVALUE='Plot Window 1')
       
       row_base = widget_base(state.w.xfindshift_base,$
                              /ROW)
   
          xmin = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='X Min:',$
                               UVALUE='X Min',$
                               XSIZE=12,$
                               EVENT_PRO='xfindshift_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.xmin_fld = [xmin,textid]
          
          xmax = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='X Max:',$
                               UVALUE='X Max',$
                               XSIZE=12,$
                               EVENT_PRO='xfindshift_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.xmax_fld = [xmax,textid]
          
          ymin = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='Y Min:',$
                               UVALUE='Y1 Min',$
                               XSIZE=12,$
                               EVENT_PRO='xfindshift_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.ymin1_fld = [ymin,textid]
          
          ymax = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='Y Max:',$
                               UVALUE='Y1 Max',$
                               XSIZE=12,$
                               EVENT_PRO='xfindshift_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.ymax1_fld = [ymax,textid]
          
          state.w.plotwin2 = widget_draw(state.w.xfindshift_base,$
                                         XSIZE=state.p.plot2size[0],$
                                         YSIZE=state.p.plot2size[1],$
                                         /TRACKING_EVENTS,$
                                         /MOTION_EVENTS,$
                                         /BUTTON_EVENTS,$
                                         EVENT_PRO='xfindshift_plotwinevent2',$
                                         UVALUE='Plot Window 2')
          
       row_base = widget_base(state.w.xfindshift_base,$
                              /ROW,$
                              EVENT_PRO='xfindshift_event')
          
          ymin = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='Y Min:',$
                               UVALUE='Y2 Min',$
                               XSIZE=12,$
                               EVENT_PRO='xfindshift_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.ymin2_fld = [ymin,textid]
          
          ymax = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='Y Max:',$
                               UVALUE='Y2 Max',$
                               XSIZE=12,$
                               EVENT_PRO='xfindshift_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.ymax2_fld = [ymax,textid]

          fix_bg = cw_bgroup(row_base,$
                             ['Fix'],$
                             FONT=buttonfont,$
                             UVALUE='Fix',$
                             /NONEXCLUSIVE)

       quit_button = widget_button(state.w.xfindshift_base,$
                                   FONT=buttonfont,$
                                   EVENT_PRO='xfindshift_event',$
                                   VALUE='Accept',$
                                   UVALUE='Accept')


; Get things running.  Center the widget using the Fanning routine.
      
    centertlb,state.w.xfindshift_base
      
    widget_control, state.w.xfindshift_base, /REALIZE
    
    mkct
    
;  Get plotwin ids
    
    widget_control, state.w.plotwin1, GET_VALUE=x
    state.p.plotwin1_wid = x
    widget_control, state.w.plotwin2, GET_VALUE=x
    state.p.plotwin2_wid = x
    
    window, /free, /pixmap,xsize=state.p.plot1size[0],$
      ysize=state.p.plot1size[1]
    state.p.pixmap1_wid = !d.window

    window, /free, /pixmap,xsize=state.p.plot2size[0],$
      ysize=state.p.plot2size[1]
    state.p.pixmap2_wid = !d.window
    
;  Get sizes for things.
    
    widget_geom = widget_info(state.w.xfindshift_base, /geometry)
    state.p.buffer[0]=widget_geom.xsize-state.p.plot1size[0]
    state.p.buffer[1]=widget_geom.ysize-state.p.plot1size[1]-$
      state.p.plot2size[1]

;  Load data

    if n_elements(INITSHIFT) ne 0 then state.r.shift = initshift

    if n_elements(OBJERROR) eq 0 or n_elements(STDERROR) eq 0 then begin

        state.r.noerror = 1
        objerror = intarr(n_elements(objflux))+1.
        stderror = intarr(n_elements(stdflux))+1.

    endif

    state.p.ytitle = (n_elements(YTITLE) ne 0) ? ytitle:''
    state.p.xtitle = (n_elements(XTITLE) ne 0) ? xtitle:''

    xfindshift_plotsetup,objwave,objflux,objerror,stdflux,stderror
    
; Start the Event Loop. This will be a non-blocking program.
    
    XManager, 'xfindshift', $
      state.w.xfindshift_base, $
      event_handler='xfindshift_resize'

    shift = state.r.shift

;    ptr_free, state.r.reg
;    ptr_free, state.d.residspec
;    ptr_free, state.d.objwave
;    ptr_free, state.d.objerror
;    ptr_free, state.d.objflux
;    ptr_free, state.d.scstdflux
;    ptr_free, state.d.scstderror
;    ptr_free, state.d.shstdflux
;    ptr_free, state.d.shstderror
;    ptr_free, state.d.stdflux
;    ptr_free, state.d.stderror

    

    cancel = state.r.cancel

;    state = 0B
    
endif

end






