; NAME:
;     xtext2spex
;
; PURPOSE:
;     To convert a text file to a FITS file xvspec can read.
;       
; CALLING SEQUENCE:
;     xtext2spex
;
; INPUTS:
;     None
;
; OUTUTS:
;     Writes a FITS file out to disk
;
; KEYWORD PARAMETERS:    
;     None
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Libray
;
; PROCEDURE:
;     Read the text file and writes out a FITS file, easy.
;
; MODIFICATION HISTORY:
;     2002-05-08 - Written by M. Cushing, Institute for Astronomy, UH
;
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xtext2spex_event, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Input Flux Units': begin

        case event.index of 

            '0': state.r.iwunits = 'W m-2 um-1'

            '1': state.r.iwunits = 'ergs s-1 cm-2 A-1'

        endcase

    end

    'Input Wavelength Units': begin

        case event.index of 

            '0': state.r.iwunits = 'um'

            '1': state.r.iwunits = 'nm'

            '2': state.r.iwunits = 'A'

        endcase

    end

    'Lines to Skip': state.r.skiplines = (event.index+1)

    'Output Flux Units': begin

        case event.index of 

            '0': state.r.owunits = 'W m-2 um-1'

            '1': state.r.owunits = 'ergs s-1 cm-2 A-1'

        endcase

    end

    'Output Wavelength Units': begin

        case event.index of 

            '0': state.r.owunits = 'um'

            '1': state.r.owunits = 'nm'

            '2': state.r.owunits = 'A'

        endcase

    end

    'Path Button': begin

        path= dialog_pickfile(/DIRECTOR,DIALOG_PARENT=state.w.xtext2spex_base,$
                              TITLE='Select Path',/MUST_EXIST)
        
        if path ne '' then begin

            path = cpath(path,WIDGET_ID=state.w.xtext2spex_base,CANCEL=cancel)
            if cancel then return
            widget_control,state.w.path_fld[1],SET_VALUE = path
            setfocus,state.w.path_fld

        endif

    end

    'Quit': begin

        state = 0B
        widget_control, event.top, /DESTROY
        goto, getout

    end

    'Text File Button': begin

        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        file = dialog_pickfile(DIALOG_PARENT=state.w.xtext2spex_base,$
                               PATH=path,/MUST_EXIST)
        
        if file eq '' then goto, cont
        widget_control,state.w.textfile_fld[1],$
          SET_VALUE = strmid(file[0],strpos(file,'/',/REVERSE_S)+1)
        setfocus,state.w.textfile_fld

    end

    'Write FITS': xtext2spex_writefits,state

endcase

;  Put state variable into the user value of the top level base.
 
cont: 
widget_control, state.w.xtext2spex_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xtext2spex_writefits,state

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return

file = cfld(state.w.textfile_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return

oname = cfld(state.w.oname_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return

rdfloat,path+file,x,y,SKIP=state.r.skiplines
ndat = n_elements(x)

if state.r.iwunits eq 'um' and state.r.owunits eq 'nm' then x = x*1000.
if state.r.iwunits eq 'um' and state.r.owunits eq 'A'  then x = x*10000.

if state.r.iwunits eq 'nm' and state.r.owunits eq 'um' then x = x/1000.
if state.r.iwunits eq 'nm' and state.r.owunits eq 'A'  then x = x*10.

if state.r.iwunits eq 'A' and state.r.owunits eq 'nm'  then x = x/10.
if state.r.iwunits eq 'A' and state.r.owunits eq 'um'  then x = x/10000.

if state.r.ifunits eq 'W m-2 um-1' and $
  state.r.ofunits eq 'ergs s-1 cm-2 A-1' then y = y/10.

if state.r.ifunits eq 'ergs s-1 cm-2 A-1' and $
  state.r.ofunits eq 'W m-2 um-1' then y = y*10.


array = [[x],[y],[fltarr(ndat)+1.0]]

mkhdr,hdr,array
fxaddpar,hdr,'NORDERS',1
fxaddpar,hdr,'NAPS',1
fxaddpar,hdr,'ORDERS','1'
fxaddpar,hdr,'XUNITS',state.r.owunits
fxaddpar,hdr,'YUNITS',state.r.ofunits

writefits,path+oname+'.fits',array,hdr




end
;
;******************************************************************************
;
; -------------------------------Main Program--------------------------------- 
;
;******************************************************************************
;
pro xtext2spex

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the operations process.
;  d - contains all of the data.

w = {buttonfont:'-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1',$
     filename_fld:[0L,0L],$
     oname_fld:[0L,0L],$
     path_fld:[0L,0L],$
     textfile_fld:[0L,0L],$
     textfont:'-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1',$
     xtext2spex_base:0L}

r = {iwunits:'um',$
     ifunits:'W m-2 um-1',$
     owunits:'um',$
     ofunits:'W m-2 um-1',$
     skiplines:0}

d = {test:1}

state = {w:w,r:r,d:d}

;  Build the widget.

state.w.xtext2spex_base = widget_base(title='Xtext2spex', $
                                      EVENT_PRO='xtext2spex_event',$
                                      /COLUMN)

   button = widget_button(state.w.xtext2spex_base,$
                          FONT=state.w.buttonfont,$
                          VALUE='Quit',$
                          UVALUE='Quit')

   box1_base = widget_base(state.w.xtext2spex_base,$
                           /COLUMN,$
                           FRAME=1)
   
      row = widget_base(box1_base,$
                        /ROW,$
                        /BASE_ALIGN_CENTER)
      
         button = widget_button(row,$
                                FONT=state.w.buttonfont,$
                                VALUE='Path',$
                                UVALUE='Path Button',$
                                EVENT_PRO='xtext2spex_event')
         
         field = coyote_field2(row,$
                               LABELFONT=state.w.buttonfont,$
                               FIELDFONT=state.w.textfont,$
                               TITLE=':',$
                               UVALUE='Path Field',$
                               XSIZE=30,$
                               TEXTID=textid)
         state.w.path_fld = [field,textid]               

      row = widget_base(box1_base,$
                        /ROW,$
                        /BASE_ALIGN_CENTER)
      
         button = widget_button(row,$
                                FONT=state.w.buttonfont,$
                                VALUE='Text File',$
                                UVALUE='Text File Button',$
                                EVENT_PRO='xtext2spex_event')
         
         field = coyote_field2(row,$
                               LABELFONT=state.w.buttonfont,$
                               FIELDFONT=state.w.textfont,$
                               TITLE=':',$
                               UVALUE='Text File Field',$
                               XSIZE=25,$
                               TEXTID=textid)
         state.w.textfile_fld = [field,textid]

         values = string(findgen(11),format='(i2.2)')
         dl = widget_droplist(box1_base,$
                              FONT=state.w.buttonfont,$
                              TITLE='Lines to Skip:',$
                              VALUE=values,$
                              UVALUE='Lines to Skip')         

   box2_base = widget_base(state.w.xtext2spex_base,$
                           /COLUMN,$
                           FRAME=1)

      dl = widget_droplist(box2_base,$
                           font=state.w.buttonfont,$
                           title='Input Wavelength Units:',$
                           value=['um','nm','A'],$
                           uvalue='Input Wavelength Units')
            
      dl = widget_droplist(box2_base,$
                           font=state.w.buttonfont,$
                           title='Input Flux Units:',$
                           value=['W m-2 um-1','ergs s-1 cm-2 A-1'],$
                           uvalue='Input Flux Units')
      
   box3_base = widget_base(state.w.xtext2spex_base,$
                           /COLUMN,$
                           FRAME=1)

      dl = widget_droplist(box3_base,$
                           FONT=state.w.buttonfont,$
                           TITLE='Output Wavelength Units:',$
                           VALUE=['um','nm','A'],$
                           UVALUE='Output Wavelength Units')
            
      dl = widget_droplist(box3_base,$
                           FONT=state.w.buttonfont,$
                           TITLE='Output Flux Units:',$
                           VALUE=['W m-2 um-1','ergs s-1 cm-2 A-1'],$
                           UVALUE='Output Flux Units')
      
   box4_base = widget_base(state.w.xtext2spex_base,$
                           /COLUMN,$
                           FRAME=1)

      fld = coyote_field2(box4_base,$
                          LABELFONT=state.w.buttonfont,$
                          FIELDFONT=state.w.textfont,$
                          TITLE='Output Name:',$
                          UVALUE='Object File Oname',$
                          XSIZE=18,$
                          TEXTID=textid)
      state.w.oname_fld = [fld,textid]
      
      button = widget_button(box4_base,$
                             FONT=state.w.buttonfont,$
                             VALUE='Write FITS',$
                             UVALUE='Write FITS')
            

; Get things running.  Center the widget using the Fanning routine.

centertlb,state.w.xtext2spex_base

widget_control, state.w.xtext2spex_base, /REALIZE

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xtext2spex', $
  state.w.xtext2spex_base, $
  /No_Block

; Put state variable into the user value of the top level base.

widget_control, state.w.xtext2spex_base, SET_UVALUE=state, /no_copy

end

